@@ -16,7 +16,7 @@ checkfail()
 # ARGUMENTS #
 #############
 
-MEDIALIBRARY_HASH=b6b64c7
+MEDIALIBRARY_HASH=a285ee80
 
 RELEASE=0
 ASAN=0
@@ -243,31 +243,25 @@ fi
 # Set up ABI variables
 if [ "${ANDROID_ABI}" = "x86" ] ; then
     TARGET_TUPLE="i686-linux-android"
-    PATH_HOST="x86"
     PLATFORM_SHORT_ARCH="x86"
 elif [ "${ANDROID_ABI}" = "x86_64" ] ; then
     TARGET_TUPLE="x86_64-linux-android"
-    PATH_HOST="x86_64"
     PLATFORM_SHORT_ARCH="x86_64"
     HAVE_64=1
 elif [ "${ANDROID_ABI}" = "mips" ] ; then
     TARGET_TUPLE="mipsel-linux-android"
-    PATH_HOST=$TARGET_TUPLE
     PLATFORM_SHORT_ARCH="mips"
 elif [ "${ANDROID_ABI}" = "mips64" ] ; then
     TARGET_TUPLE="mips64el-linux-android"
-    PATH_HOST=$TARGET_TUPLE
     PLATFORM_SHORT_ARCH="mips64"
     HAVE_64=1
 elif [ "${ANDROID_ABI}" = "arm64-v8a" ] ; then
     TARGET_TUPLE="aarch64-linux-android"
-    PATH_HOST=$TARGET_TUPLE
     HAVE_ARM=1
     HAVE_64=1
     PLATFORM_SHORT_ARCH="arm64"
 elif [ "${ANDROID_ABI}" = "armeabi-v7a" ] ; then
     TARGET_TUPLE="arm-linux-androideabi"
-    PATH_HOST=$TARGET_TUPLE
     HAVE_ARM=1
     PLATFORM_SHORT_ARCH="arm"
 else
@@ -399,9 +393,10 @@ EXTRA_CXXFLAGS="${EXTRA_CXXFLAGS} -D__STDC_FORMAT_MACROS=1 -D__STDC_CONSTANT_MAC
 #################
 
 VLC_LDFLAGS=""
+EXTRA_LDFLAGS=""
 if [ ${ANDROID_ABI} = "armeabi-v7a" ]; then
         EXTRA_PARAMS=" --enable-neon"
-        VLC_LDFLAGS="${VLC_LDFLAGS} -Wl,--fix-cortex-a8"
+        EXTRA_LDFLAGS="${EXTRA_LDFLAGS} -Wl,--fix-cortex-a8"
 fi
 NDK_LIB_DIR="${NDK_TOOLCHAIN_DIR}/${TARGET_TUPLE}/lib"
 if [ "${PLATFORM_SHORT_ARCH}" = "x86_64" -o "${PLATFORM_SHORT_ARCH}" = "mips64" ];then
@@ -413,7 +408,8 @@ if [ "${ANDROID_ABI}" = "armeabi-v7a" ];then
     NDK_LIB_UNWIND="-lunwind"
 fi
 
-VLC_LDFLAGS="${VLC_LDFLAGS} -L${NDK_LIB_DIR} -lc++abi ${NDK_LIB_UNWIND}"
+EXTRA_LDFLAGS="${EXTRA_LDFLAGS} -L${NDK_LIB_DIR} -lc++abi ${NDK_LIB_UNWIND}"
+VLC_LDFLAGS="${EXTRA_LDFLAGS}"
 
 # Release or not?
 if [ "$RELEASE" = 1 ]; then
@@ -515,6 +511,7 @@ checkfail "contribs: bootstrap failed"
 
 echo "EXTRA_CFLAGS=${EXTRA_CFLAGS}" >> config.mak
 echo "EXTRA_CXXFLAGS=${EXTRA_CXXFLAGS}" >> config.mak
+echo "EXTRA_LDFLAGS=${EXTRA_LDFLAGS}" >> config.mak
 
 make $MAKEFLAGS fetch
 checkfail "contribs: make fetch failed"
@@ -755,13 +752,22 @@ fi
 MEDIALIBRARY_MODULE_DIR=${SRC_DIR}/medialibrary
 MEDIALIBRARY_BUILD_DIR=${MEDIALIBRARY_MODULE_DIR}/medialibrary
 OUT_LIB_DIR=$MEDIALIBRARY_MODULE_DIR/jni/libs/${ANDROID_ABI}
-SQLITE_RELEASE="sqlite-autoconf-3120200"
+SQLITE_RELEASE="sqlite-autoconf-3160200"
+SQLITE_SHA1="64ca578ad44a94115b1db0406740e14288f74bb8"
 
 if [ ! -d "${MEDIALIBRARY_MODULE_DIR}/${SQLITE_RELEASE}" ]; then
     echo -e "\e[1m\e[32msqlite source not found, downloading\e[0m"
     cd ${MEDIALIBRARY_MODULE_DIR}
-    wget https://www.sqlite.org/2016/${SQLITE_RELEASE}.tar.gz
+    rm -rf ${MEDIALIBRARY_BUILD_DIR}/build*
+    rm -rf ${MEDIALIBRARY_MODULE_DIR}/jni/libs
+    rm -rf ${MEDIALIBRARY_MODULE_DIR}/jni/obj
+    wget https://download.videolan.org/pub/contrib/sqlite/${SQLITE_RELEASE}.tar.gz
+    if [ ! "`sha1sum ${SQLITE_RELEASE}.tar.gz`" = "${SQLITE_SHA1}  ${SQLITE_RELEASE}.tar.gz" ]; then
+        echo "Wrong sha1 for ${SQLITE_RELEASE}.tar.gz"
+        exit 1
+    fi
     tar -xzf ${SQLITE_RELEASE}.tar.gz
+    rm -f ${SQLITE_RELEASE}.tar.gz
 fi
 cd ${MEDIALIBRARY_MODULE_DIR}/${SQLITE_RELEASE}
 if [ ! -d "build-$ANDROID_ABI" ]; then
