@@ -22,8 +22,21 @@
 
 package org.videolan.libvlc;
 
+import android.annotation.TargetApi;
+import android.content.BroadcastReceiver;
+import android.content.Context;
+import android.content.Intent;
+import android.content.IntentFilter;
+import android.media.AudioDeviceCallback;
+import android.media.AudioDeviceInfo;
+import android.media.AudioFormat;
+import android.media.AudioManager;
 import android.net.Uri;
+import android.os.Build;
+import android.support.annotation.RequiresApi;
+import android.util.SparseArray;
 
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.libvlc.util.VLCUtil;
 
 import java.io.File;
@@ -336,11 +349,15 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
     private Media mMedia = null;
     private boolean mPlaying = false;
     private boolean mPlayRequested = false;
+    private boolean mAudioDeviceFromUser = false;
     private int mVoutCount = 0;
     private boolean mAudioReset = false;
-    private String mAudioOutput = null;
+    private String mAudioOutput = "android_audiotrack";
     private String mAudioOutputDevice = null;
 
+    private boolean mAudioPlugRegistered = false;
+    private String mAudioPlugOutputDevice = "stereo";
+
     private final AWindow mWindow = new AWindow(new AWindow.SurfaceCallback() {
         @Override
         public void onSurfacesCreated(AWindow vout) {
@@ -370,12 +387,141 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
         }
     });
 
+    private void updateAudioOutputDevice(long encodingFlags, String defaultDevice) {
+        final String newDeviceId = encodingFlags != 0 ? "encoded:" + encodingFlags : defaultDevice;
+        if (!newDeviceId.equals(mAudioPlugOutputDevice)) {
+            mAudioPlugOutputDevice = newDeviceId;
+            setAudioOutputDeviceInternal(mAudioPlugOutputDevice, false);
+        }
+    }
+
+    private boolean isEncoded(int encoding) {
+        switch (encoding) {
+            case AudioFormat.ENCODING_AC3:
+            case AudioFormat.ENCODING_E_AC3:
+            case 14 /* AudioFormat.ENCODING_DOLBY_TRUEHD */:
+            case AudioFormat.ENCODING_DTS:
+            case AudioFormat.ENCODING_DTS_HD:
+                return true;
+            default:
+                return false;
+        }
+    }
+
+    private long getEncodingFlags(int encodings[]) {
+        long encodingFlags = 0;
+        for (int encoding : encodings) {
+            if (isEncoded(encoding))
+                encodingFlags |= 1 << encoding;
+        }
+        return encodingFlags;
+    }
+
+    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
+    private BroadcastReceiver createAudioPlugReceiver() {
+        return new BroadcastReceiver() {
+            @Override
+            public void onReceive(Context context, Intent intent) {
+                final String action = intent.getAction();
+                if (action == null)
+                    return;
+                if (action.equalsIgnoreCase(AudioManager.ACTION_HDMI_AUDIO_PLUG)) {
+                    final boolean hasHdmi = intent.getIntExtra(AudioManager.EXTRA_AUDIO_PLUG_STATE, 0) == 1;
+                    final long encodingFlags = !hasHdmi ? 0 :
+                            getEncodingFlags(intent.getIntArrayExtra(AudioManager.EXTRA_ENCODINGS));
+                    updateAudioOutputDevice(encodingFlags, "stereo");
+                }
+            }
+        };
+    }
+
+    private final BroadcastReceiver mAudioPlugReceiver =
+            AndroidUtil.isLolliPopOrLater && !AndroidUtil.isMarshMallowOrLater ? createAudioPlugReceiver() : null;
+
+    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
+    private void registerAudioPlugV21(boolean register) {
+        if (register) {
+            final IntentFilter intentFilter = new IntentFilter(AudioManager.ACTION_HDMI_AUDIO_PLUG);
+            final Intent stickyIntent = mLibVLC.mAppContext.registerReceiver(mAudioPlugReceiver, intentFilter);
+            if (stickyIntent != null)
+                mAudioPlugReceiver.onReceive(mLibVLC.mAppContext, stickyIntent);
+        } else {
+            mLibVLC.mAppContext.unregisterReceiver(mAudioPlugReceiver);
+        }
+    }
+
+    @TargetApi(Build.VERSION_CODES.M)
+    private AudioDeviceCallback createAudioDeviceCallback() {
+
+        return new AudioDeviceCallback() {
+
+            private SparseArray<Long> mEncodedDevices = new SparseArray<>();
+
+            private void onAudioDevicesChanged() {
+                long encodingFlags = 0;
+                for (int i = 0; i < mEncodedDevices.size(); ++i)
+                    encodingFlags |= mEncodedDevices.valueAt(i);
+
+                updateAudioOutputDevice(encodingFlags, "pcm");
+            }
+
+            @RequiresApi(Build.VERSION_CODES.M)
+            @Override
+            public void onAudioDevicesAdded(AudioDeviceInfo[] addedDevices) {
+                for (AudioDeviceInfo info : addedDevices) {
+                    if (!info.isSink())
+                        continue;
+                    long encodingFlags = getEncodingFlags(info.getEncodings());
+                    if (encodingFlags != 0)
+                        mEncodedDevices.put(info.getId(), encodingFlags);
+                }
+                onAudioDevicesChanged();
+            }
+
+            @RequiresApi(Build.VERSION_CODES.M)
+            @Override
+            public void onAudioDevicesRemoved(AudioDeviceInfo[] removedDevices) {
+                for (AudioDeviceInfo info : removedDevices) {
+                    if (!info.isSink())
+                        continue;
+                    mEncodedDevices.remove(info.getId());
+                }
+                onAudioDevicesChanged();
+            }
+        };
+    }
+
+    private final AudioDeviceCallback mAudioDeviceCallback =
+            AndroidUtil.isMarshMallowOrLater ? createAudioDeviceCallback() : null;
+
+    @TargetApi(Build.VERSION_CODES.M)
+    private void registerAudioPlugV23(boolean register) {
+        AudioManager am = (AudioManager) mLibVLC.mAppContext.getSystemService(Context.AUDIO_SERVICE);
+        if (register) {
+            mAudioDeviceCallback.onAudioDevicesAdded(am.getDevices(AudioManager.GET_DEVICES_OUTPUTS));
+            am.registerAudioDeviceCallback(mAudioDeviceCallback, null);
+        } else {
+            am.unregisterAudioDeviceCallback(mAudioDeviceCallback);
+        }
+    }
+
+    private void registerAudioPlug(boolean register) {
+        if (register == mAudioPlugRegistered)
+            return;
+        if (mAudioDeviceCallback != null)
+            registerAudioPlugV23(register);
+        else if (mAudioPlugReceiver != null)
+            registerAudioPlugV21(register);
+        mAudioPlugRegistered = register;
+    }
+
     /**
      * Create an empty MediaPlayer
      *
      * @param libVLC a valid libVLC
      */
     public MediaPlayer(LibVLC libVLC) {
+        super(libVLC);
         nativeNewFromLibVlc(libVLC, mWindow);
     }
 
@@ -385,6 +531,7 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
      * @param media a valid Media object
      */
     public MediaPlayer(Media media) {
+        super(media);
         if (media == null || media.isReleased())
             throw new IllegalArgumentException("Media is null or released");
         mMedia = media;
@@ -445,6 +592,8 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
                         nativeSetAudioOutputDevice(mAudioOutputDevice);
                     mAudioReset = false;
                 }
+                if (!mAudioDeviceFromUser)
+                    registerAudioPlug(true);
                 mPlayRequested = true;
                 if (mWindow.areSurfacesWaiting())
                     return;
@@ -538,6 +687,11 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
      * Any change will take effect only after playback is stopped and
      * restarted. Audio output cannot be changed while playing.
      *
+     * By default, the "android_audiotrack" is selected. Starting Android 21, passthrough is
+     * enabled for encodings supported by the device/audio system.
+     *
+     * Calling this method will disable the encoding detection.
+     *
      * @return true on success.
      */
     public boolean setAudioOutput(String aout) {
@@ -545,27 +699,49 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
         if (ret) {
             synchronized (this) {
                 mAudioOutput = aout;
+                /* The user forced an output, don't listen to audio plug events and let the user decide */
+                mAudioDeviceFromUser = true;
+                registerAudioPlug(false);
             }
         }
         return ret;
     }
 
-    /**
-     * Configures an explicit audio output device.
-     * Audio output will be moved to the device specified by the device identifier string.
-     *
-     * @return true on success.
-     */
-    public boolean setAudioOutputDevice(String id) {
+    private boolean setAudioOutputDeviceInternal(String id, boolean fromUser) {
         final boolean ret = nativeSetAudioOutputDevice(id);
         if (ret) {
             synchronized (this) {
                 mAudioOutputDevice = id;
+                if (fromUser) {
+                    /* The user forced a device, don't listen to audio plug events and let the user decide */
+                    mAudioDeviceFromUser = true;
+                    registerAudioPlug(false);
+                }
             }
         }
         return ret;
     }
 
+        /**
+         * Configures an explicit audio output device.
+         * Audio output will be moved to the device specified by the device identifier string.
+         *
+         * Available devices for the "android_audiotrack" module (the default) are
+         * "stereo": Up to 2 channels (compat mode).
+         * "pcm": Up to 8 channels.
+         * "encoded": Up to 8 channels, passthrough for every encodings if available.
+         * "encoded:ENCODING_FLAGS_MASK": passthrough for every encodings specified by
+         * ENCODING_FLAGS_MASK. This extra value is a long that contains binary-shifted
+         * AudioFormat.ENCODING_* values.
+         *
+         * Calling this method will disable the encoding detection (see {@link #setAudioOutput}).
+         *
+         * @return true on success.
+         */
+    public boolean setAudioOutputDevice(String id) {
+        return setAudioOutputDeviceInternal(id, true);
+    }
+
     /**
      * Get the full description of available titles.
      *
@@ -934,6 +1110,8 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
 
     @Override
     protected void onReleaseNative() {
+        registerAudioPlug(false);
+
         if (mMedia != null)
             mMedia.release();
         nativeRelease();
