@@ -24,6 +24,7 @@ import android.annotation.TargetApi;
 import android.app.KeyguardManager;
 import android.app.Notification;
 import android.app.PendingIntent;
+import android.app.SearchManager;
 import android.content.BroadcastReceiver;
 import android.content.ComponentName;
 import android.content.Context;
@@ -71,7 +72,10 @@ import org.videolan.libvlc.MediaList;
 import org.videolan.libvlc.MediaPlayer;
 import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.medialibrary.Medialibrary;
+import org.videolan.medialibrary.Tools;
+import org.videolan.medialibrary.media.MediaLibraryItem;
 import org.videolan.medialibrary.media.MediaWrapper;
+import org.videolan.medialibrary.media.SearchAggregate;
 import org.videolan.vlc.gui.AudioPlayerContainerActivity;
 import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.BitmapUtil;
@@ -88,6 +92,7 @@ import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Strings;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.VLCOptions;
+import org.videolan.vlc.util.VoiceSearchParams;
 import org.videolan.vlc.util.WeakHandler;
 import org.videolan.vlc.widget.VLCAppWidgetProvider;
 
@@ -105,8 +110,6 @@ import java.util.Random;
 import java.util.Stack;
 import java.util.concurrent.atomic.AtomicBoolean;
 
-import static android.R.attr.id;
-
 public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVout.Callback {
 
     private static final String TAG = "VLC/PlaybackService";
@@ -123,6 +126,9 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
     public static final String ACTION_REMOTE_LAST_PLAYLIST = ACTION_REMOTE_GENERIC+"LastPlaylist";
     public static final String ACTION_REMOTE_LAST_VIDEO_PLAYLIST = ACTION_REMOTE_GENERIC+"LastVideoPlaylist";
     public static final String ACTION_REMOTE_SWITCH_VIDEO = ACTION_REMOTE_GENERIC+"SwitchToVideo";
+    public static final String ACTION_PLAY_FROM_SEARCH = ACTION_REMOTE_GENERIC+"play_from_search";
+
+    public static final String EXTRA_SEARCH_BUNDLE = ACTION_REMOTE_GENERIC+"extra_search_bundle";
 
     private static final int DELAY_DOUBLE_CLICK = 800;
     private static final int DELAY_LONG_CLICK = 1000;
@@ -152,18 +158,14 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
     private boolean mParsed = false;
     private boolean mSeekable = false;
     private boolean mPausable = false;
-    private boolean mIsAudioTrack = false;
-    private boolean mHasHdmiAudio = false;
     private boolean mSwitchingToVideo = false;
     private boolean mVideoBackground = false;
 
-    final private ArrayList<Callback> mCallbacks = new ArrayList<Callback>();
+    final private ArrayList<Callback> mCallbacks = new ArrayList<>();
     private boolean mDetectHeadset = true;
     private PowerManager.WakeLock mWakeLock;
     private final AtomicBoolean mExpanding = new AtomicBoolean(false);
 
-    private static boolean mWasPlayingAudio = false; // used only if readPhoneState returns true
-
     // Index management
     /**
      * Stack of previously played indexes, used in shuffle mode
@@ -216,12 +218,8 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
     private MediaPlayer newMediaPlayer() {
         final MediaPlayer mp = new MediaPlayer(LibVLC());
         final String aout = VLCOptions.getAout(mSettings);
-        if (mp.setAudioOutput(aout) && aout.equals("android_audiotrack")) {
-            mIsAudioTrack = true;
-            if (mHasHdmiAudio)
-                mp.setAudioOutputDevice("hdmi");
-        } else
-            mIsAudioTrack = false;
+        if (aout != null)
+            mp.setAudioOutput(aout);
         mp.getVLCVout().addCallback(this);
 
         return mp;
@@ -277,7 +275,6 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
         filter.addAction(AudioManager.ACTION_AUDIO_BECOMING_NOISY);
         filter.addAction(VLCApplication.SLEEP_INTENT);
         registerReceiver(mReceiver, filter);
-        registerV21();
 
         boolean stealRemoteControl = mSettings.getBoolean("enable_steal_remote_control", false);
 
@@ -296,16 +293,23 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
     public int onStartCommand(Intent intent, int flags, int startId) {
         if (intent == null)
             return START_STICKY;
-        if(ACTION_REMOTE_PLAYPAUSE.equals(intent.getAction())){
+        String action = intent.getAction();
+        if (ACTION_REMOTE_PLAYPAUSE.equals(action)){
             if (hasCurrentMedia())
                 return START_STICKY;
             else
                 loadLastPlaylist(TYPE_AUDIO);
-        } else if (ACTION_REMOTE_PLAY.equals(intent.getAction())) {
+        } else if (ACTION_REMOTE_PLAY.equals(action)) {
             if (hasCurrentMedia())
                 play();
             else
                 loadLastPlaylist(TYPE_AUDIO);
+        } else if (ACTION_PLAY_FROM_SEARCH.equals(action)) {
+            if (mMediaSession == null)
+                initMediaSession();
+            Bundle extras = intent.getBundleExtra(EXTRA_SEARCH_BUNDLE);
+            mMediaSession.getController().getTransportControls()
+                    .playFromSearch(extras.getString(SearchManager.QUERY), extras);
         }
         updateWidget();
         return super.onStartCommand(intent, flags, startId);
@@ -327,8 +331,6 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
         if (mWakeLock.isHeld())
             mWakeLock.release();
         unregisterReceiver(mReceiver);
-        if (mReceiverV21 != null)
-            unregisterReceiver(mReceiverV21);
         if (mRemoteControlClientReceiver != null) {
             unregisterReceiver(mRemoteControlClientReceiver);
             mRemoteControlClientReceiver = null;
@@ -409,29 +411,6 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
         }
     }
 
-
-    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
-    private void registerV21() {
-        final IntentFilter intentFilter = new IntentFilter(AudioManager.ACTION_HDMI_AUDIO_PLUG);
-        registerReceiver(mReceiverV21, intentFilter);
-    }
-
-    private final BroadcastReceiver mReceiverV21 = AndroidUtil.isLolliPopOrLater() ? new BroadcastReceiver()
-    {
-        @TargetApi(Build.VERSION_CODES.LOLLIPOP)
-        @Override
-        public void onReceive(Context context, Intent intent) {
-            final String action = intent.getAction();
-            if (action == null)
-                return;
-            if (action.equalsIgnoreCase(AudioManager.ACTION_HDMI_AUDIO_PLUG)) {
-                mHasHdmiAudio = intent.getIntExtra(AudioManager.EXTRA_AUDIO_PLUG_STATE, 0) == 1;
-                if (mMediaPlayer != null && mIsAudioTrack)
-                    mMediaPlayer.setAudioOutputDevice(mHasHdmiAudio ? "hdmi" : "stereo");
-            }
-        }
-    } : null;
-
     private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
         private boolean wasPlaying = false;
         @Override
@@ -495,9 +474,9 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
             /*
              * headset plug events
              */
-            else if (mDetectHeadset && !mHasHdmiAudio) {
+            else if (mDetectHeadset) {
                 if (action.equalsIgnoreCase(AudioManager.ACTION_AUDIO_BECOMING_NOISY)) {
-                    Log.i(TAG, "Headset Removed.");
+                    Log.i(TAG, "Becoming noisy");
                     wasPlaying = isPlaying();
                     if (wasPlaying && hasCurrentMedia())
                         pause();
@@ -541,8 +520,10 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
 
             }
             if (update) {
-                for (Callback callback : mCallbacks)
-                    callback.onMediaEvent(event);
+                synchronized (mCallbacks) {
+                    for (Callback callback : mCallbacks)
+                        callback.onMediaEvent(event);
+                }
                 if (mParsed && mMediaSession != null)
                     showNotification();
             }
@@ -665,12 +646,14 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
                 case MediaPlayer.Event.MediaChanged:
                     Log.d(TAG, "onEvent: MediaChanged");
             }
-            for (Callback callback : mCallbacks)
-                callback.onMediaPlayerEvent(event);
+            synchronized (mCallbacks) {
+                for (Callback callback : mCallbacks)
+                    callback.onMediaPlayerEvent(event);
+            }
         }
     };
 
-    private void saveMediaMeta() {
+    public void saveMediaMeta() {
         MediaWrapper media = mMedialibrary.findMedia(getCurrentMediaWrapper());
         if (media == null || media.getId() == 0)
             return;
@@ -786,8 +769,10 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
     }
 
     private void executeUpdate(Boolean updateWidget) {
-        for (Callback callback : mCallbacks) {
-            callback.update();
+        synchronized (mCallbacks) {
+            for (Callback callback : mCallbacks) {
+                callback.update();
+            }
         }
         if (updateWidget)
             updateWidget();
@@ -796,8 +781,10 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
     }
 
     private void executeUpdateProgress() {
-        for (Callback callback : mCallbacks) {
-            callback.updateProgress();
+        synchronized (mCallbacks) {
+            for (Callback callback : mCallbacks) {
+                callback.updateProgress();
+            }
         }
     }
 
@@ -830,14 +817,17 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
         @Override
         public void handleMessage(Message msg) {
             PlaybackService service = getOwner();
-            if(service == null) return;
+            if (service == null)
+                return;
 
             switch (msg.what) {
                 case SHOW_PROGRESS:
-                    if (service.mCallbacks.size() > 0) {
-                        removeMessages(SHOW_PROGRESS);
-                        service.executeUpdateProgress();
-                        sendEmptyMessageDelayed(SHOW_PROGRESS, 1000);
+                    synchronized (service.mCallbacks) {
+                        if (service.mCallbacks.size() > 0) {
+                            removeMessages(SHOW_PROGRESS);
+                            service.executeUpdateProgress();
+                            sendEmptyMessageDelayed(SHOW_PROGRESS, 1000);
+                        }
                     }
                     break;
                 case SHOW_TOAST:
@@ -903,7 +893,7 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
             if (AndroidDevices.showMediaStyle) {
                 builder.setStyle(new NotificationCompat.MediaStyle()
                                 .setMediaSession(mMediaSession.getSessionToken())
-                                .setShowActionsInCompactView(new int[] {0,1,2})
+                                .setShowActionsInCompactView(0,1,2)
                                 .setShowCancelButton(true)
                                 .setCancelButtonIntent(piStop)
                 );
@@ -912,7 +902,7 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
             notification = builder.build();
 
             startService(new Intent(this, PlaybackService.class));
-            if (!AndroidUtil.isLolliPopOrLater() || mMediaPlayer.isPlaying())
+            if (!AndroidUtil.isLolliPopOrLater || mMediaPlayer.isPlaying())
                 startForeground(3, notification);
             else {
                 stopForeground(false);
@@ -1189,6 +1179,40 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
             loadUri(uri);
         }
 
+        @Override
+        public void onPlayFromSearch(String query, Bundle extras) {
+            mMediaSession.setPlaybackState(new PlaybackStateCompat.Builder().setState(PlaybackStateCompat.STATE_CONNECTING, getTime(), 1.0f).build());
+            VoiceSearchParams vsp = new VoiceSearchParams(query, extras);
+            MediaLibraryItem[] items = null;
+            MediaWrapper[] tracks = null;
+            if (vsp.isAny) {
+                items = mMedialibrary.getAudio();
+                if (!isShuffling())
+                    shuffle();
+            } else if (vsp.isArtistFocus) {
+                items = mMedialibrary.searchArtist(vsp.artist);
+            } else if (vsp.isAlbumFocus) {
+                items = mMedialibrary.searchAlbum(vsp.album);
+            } else if (vsp.isGenreFocus) {
+                items = mMedialibrary.searchGenre(vsp.genre);
+            } else if (vsp.isSongFocus) {
+                tracks = mMedialibrary.searchMedia(vsp.song).getTracks();
+            }
+            if (Tools.isArrayEmpty(tracks)){
+                SearchAggregate result = mMedialibrary.search(query);
+                if (!Tools.isArrayEmpty(result.getAlbums()))
+                    tracks = result.getAlbums()[0].getTracks(mMedialibrary);
+                else if (!Tools.isArrayEmpty(result.getArtists()))
+                    tracks = result.getArtists()[0].getTracks(mMedialibrary);
+                else if (!Tools.isArrayEmpty(result.getGenres()))
+                    tracks = result.getGenres()[0].getTracks(mMedialibrary);
+            }
+            if (tracks == null && !Tools.isArrayEmpty(items))
+                tracks = items[0].getTracks(mMedialibrary);
+            if (!Tools.isArrayEmpty(tracks))
+                load(tracks, 0);
+        }
+
         @Override
         public void onPause() {
             pause();
@@ -1685,16 +1709,20 @@ public class PlaybackService extends MediaBrowserServiceCompat implements IVLCVo
 
     @MainThread
     public synchronized void addCallback(Callback cb) {
-        if (!mCallbacks.contains(cb)) {
-            mCallbacks.add(cb);
-            if (hasCurrentMedia())
-            mHandler.sendEmptyMessage(SHOW_PROGRESS);
+        synchronized (mCallbacks) {
+            if (!mCallbacks.contains(cb)) {
+                mCallbacks.add(cb);
+                if (hasCurrentMedia())
+                    mHandler.sendEmptyMessage(SHOW_PROGRESS);
+            }
         }
     }
 
     @MainThread
     public synchronized void removeCallback(Callback cb) {
-        mCallbacks.remove(cb);
+        synchronized (mCallbacks) {
+            mCallbacks.remove(cb);
+        }
     }
 
     @MainThread
