@@ -56,6 +56,7 @@ public class VLCApplication extends Application {
 
     public final static String ACTION_MEDIALIBRARY_READY = "VLC/VLCApplication";
     private static VLCApplication instance;
+    private static Medialibrary sMedialibraryInstance;
 
     public final static String SLEEP_INTENT = Strings.buildPkgString("SleepIntent");
 
@@ -67,7 +68,8 @@ public class VLCApplication extends Application {
     private static SimpleArrayMap<String, Object> sDataMap = new SimpleArrayMap<>();
 
     /* Up to 2 threads maximum, inactive threads are killed after 2 seconds */
-    private ThreadPoolExecutor mThreadPool = new ThreadPoolExecutor(2, AndroidUtil.isJellyBeanMR1OrLater() ? Runtime.getRuntime().availableProcessors() : 2, 2, TimeUnit.SECONDS,
+    int maxThreads = Math.max(AndroidUtil.isJellyBeanMR1OrLater ? Runtime.getRuntime().availableProcessors() : 2, 1);
+    private ThreadPoolExecutor mThreadPool = new ThreadPoolExecutor(Math.min(2, maxThreads), maxThreads, 2, TimeUnit.SECONDS,
             new LinkedBlockingQueue<Runnable>(), THREAD_FACTORY);
     public static final ThreadFactory THREAD_FACTORY = new ThreadFactory() {
         @Override
@@ -195,6 +197,10 @@ public class VLCApplication extends Application {
         return sDataMap.remove(key);
     }
 
+    public static void clearData() {
+        sDataMap.clear();
+    }
+
     Dialog.Callbacks mDialogCallbacks = new Dialog.Callbacks() {
         @Override
         public void onDisplay(Dialog.ErrorMessage dialog) {
@@ -221,7 +227,7 @@ public class VLCApplication extends Application {
 
         @Override
         public void onCanceled(Dialog dialog) {
-            if (dialog != null)
+            if (dialog != null && dialog.getContext() != null)
                 ((DialogFragment)dialog.getContext()).dismiss();
         }
 
@@ -240,7 +246,10 @@ public class VLCApplication extends Application {
     }
 
     public static synchronized Medialibrary getMLInstance() {
-        VLCInstance.get(); // ensure VLC is loaded before medialibrary
-        return Medialibrary.getInstance();
+        if (sMedialibraryInstance == null) {
+            VLCInstance.get(); // ensure VLC is loaded before medialibrary
+            sMedialibraryInstance = Medialibrary.getInstance();
+        }
+        return sMedialibraryInstance;
     }
 }
