@@ -75,7 +75,7 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Random;
 
-public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDiscoveryCb, SwipeRefreshLayout.OnRefreshListener, MediaBrowser.EventListener, ViewPager.OnPageChangeListener, Medialibrary.ArtistsAddedCb, Medialibrary.ArtistsModifiedCb, Medialibrary.AlbumsAddedCb, Medialibrary.AlbumsModifiedCb, MediaAddedCb, MediaUpdatedCb, TabLayout.OnTabSelectedListener, Filterable {
+public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDiscoveryCb, SwipeRefreshLayout.OnRefreshListener, MediaBrowser.EventListener, ViewPager.OnPageChangeListener, Medialibrary.ArtistsAddedCb, Medialibrary.ArtistsModifiedCb, Medialibrary.AlbumsAddedCb, Medialibrary.AlbumsModifiedCb, MediaAddedCb, MediaUpdatedCb, TabLayout.OnTabSelectedListener, Filterable, View.OnClickListener {
     public final static String TAG = "VLC/AudioBrowserFragment";
 
     private MediaBrowser mMediaBrowser;
@@ -132,6 +132,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDis
         View v = inflater.inflate(R.layout.audio_browser, container, false);
 
         mEmptyView = (TextView) v.findViewById(R.id.no_media);
+        mEmptyView.setOnClickListener(this);
 
         mViewPager = (ViewPager) v.findViewById(R.id.pager);
         mFastScroller = (FastScroller) v.findViewById(R.id.songs_fast_scroller);
@@ -293,10 +294,10 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDis
             String message;
             Runnable action;
             final MediaLibraryItem separator = previous != null && previous.getItemType() == MediaLibraryItem.TYPE_DUMMY &&
-                    (next == null || next.getItemType() == MediaLibraryItem.TYPE_DUMMY) ? adapter.getItem(position-1) : null;
-            adapter.remove(position);
+                    (next == null || next.getItemType() == MediaLibraryItem.TYPE_DUMMY) ? previous : null;
+            adapter.remove(mediaLibraryItem);
             if (separator != null)
-                adapter.remove(position-1);
+                adapter.remove(separator);
 
             if (mode == MODE_PLAYLIST) {
                 message = getString(R.string.playlist_deleted);
@@ -340,7 +341,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDis
         }
 
         if (id == R.id.audio_view_add_playlist) {
-            UiTools.addToPlaylist(getActivity(), new ArrayList<>(Arrays.asList(mediaItem.getTracks(mMediaLibrary))));
+            UiTools.addToPlaylist(getActivity(), mediaItem.getTracks(mMediaLibrary));
             return true;
         }
 
@@ -559,7 +560,8 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDis
     @Override
     public void onUpdateFinished(RecyclerView.Adapter adapter) {
         if (adapter == getCurrentAdapter()) {
-            mHandler.sendEmptyMessage(UNSET_REFRESHING);
+            if (!mMediaLibrary.isWorking())
+                mHandler.sendEmptyMessage(UNSET_REFRESHING);
             mSwipeRefreshLayout.setEnabled(((LinearLayoutManager)getCurrentRV().getLayoutManager()).findFirstVisibleItemPosition() <= 0);
             updateEmptyView(mViewPager.getCurrentItem());
             mFastScroller.setRecyclerView(getCurrentRV());
@@ -670,6 +672,15 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDis
         return mLists[mViewPager.getCurrentItem()];
     }
 
+    @Override
+    public void onClick(View v) {
+        if (v.getId() == R.id.no_media) {
+            Intent intent = new Intent(v.getContext(), SecondaryActivity.class);
+            intent.putExtra("fragment", SecondaryActivity.STORAGE_BROWSER);
+            startActivity(intent);
+        }
+    }
+
     private static class AudioBrowserHandler extends WeakHandler<AudioBrowserFragment> {
     AudioBrowserHandler(AudioBrowserFragment owner) {
         super(owner);
@@ -696,6 +707,8 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements DevicesDis
                 fragment.mSwipeRefreshLayout.setRefreshing(true);
                 break;
             case UNSET_REFRESHING:
+                removeMessages(SET_REFRESHING);
+                removeMessages(MSG_LOADING);
                 fragment.mSwipeRefreshLayout.setRefreshing(false);
                 break;
             case UPDATE_EMPTY_VIEW:
