@@ -36,8 +36,10 @@ import android.os.Message;
 import android.preference.PreferenceManager;
 import android.support.annotation.Nullable;
 import android.support.annotation.RequiresPermission;
+import android.support.constraint.ConstraintSet;
 import android.support.design.widget.BottomSheetBehavior;
 import android.support.design.widget.Snackbar;
+import android.support.transition.TransitionManager;
 import android.support.v4.app.FragmentActivity;
 import android.support.v4.app.FragmentManager;
 import android.support.v7.widget.LinearLayoutManager;
@@ -96,6 +98,8 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
     private boolean mHeaderTimeVisible;
     private int mPlayerState;
     private String mCurrentCoverArt;
+    private final ConstraintSet coverConstraintSet = AndroidUtil.isICSOrLater ? new ConstraintSet() : null;
+    private final ConstraintSet playlistConstraintSet = AndroidUtil.isICSOrLater ? new ConstraintSet() : null;
 
     // Tips
     private static final String PREF_PLAYLIST_TIPS_SHOWN = "playlist_tips_shown";
@@ -110,7 +114,7 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
     @Override
     public void onViewCreated(final View view, @Nullable Bundle savedInstanceState) {
         super.onViewCreated(view, savedInstanceState);
-        if (AndroidUtil.isJellyBeanMR1OrLater()) {
+        if (AndroidUtil.isJellyBeanMR1OrLater) {
             DEFAULT_BACKGROUND_DARKER_ID = UiTools.getResourceFromAttribute(view.getContext(), R.attr.background_default_darker);
             DEFAULT_BACKGROUND_ID = UiTools.getResourceFromAttribute(view.getContext(), R.attr.background_default);
         }
@@ -138,6 +142,12 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
         registerForContextMenu(mBinding.songsList);
         getActivity().setVolumeControlStream(AudioManager.STREAM_MUSIC);
         setUserVisibleHint(true);
+        if (playlistConstraintSet != null) {
+            playlistConstraintSet.clone(mBinding.contentLayout);
+            coverConstraintSet.clone(mBinding.contentLayout);
+            coverConstraintSet.setVisibility(R.id.songs_list, View.GONE);
+            coverConstraintSet.setVisibility(R.id.cover_media_switcher, View.VISIBLE);
+        }
     }
 
     public void onPopupMenu(View anchor, final int position) {
@@ -192,7 +202,7 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
         if (mService.hasMedia() && !mService.isVideoPlaying()) {
             SharedPreferences mSettings= PreferenceManager.getDefaultSharedPreferences(getActivity());
             //Check fragment resumed to not restore video on device turning off
-            if (isResumed() && mSettings.getBoolean(PreferencesActivity.VIDEO_RESTORE, false)) {
+            if (isVisible() && mSettings.getBoolean(PreferencesActivity.VIDEO_RESTORE, false)) {
                 mSettings.edit().putBoolean(PreferencesActivity.VIDEO_RESTORE, false).apply();
                 mService.getCurrentMediaWrapper().removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
                 mService.switchToVideo();
@@ -275,14 +285,13 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
     }
 
     private void updateBackground() {
-        if (AndroidUtil.isJellyBeanMR1OrLater()) {
+        if (AndroidUtil.isJellyBeanMR1OrLater) {
             final MediaWrapper mw = mService.getCurrentMediaWrapper();
             if (mw == null || TextUtils.equals(mCurrentCoverArt, mw.getArtworkMrl()))
                 return;
             mCurrentCoverArt = mw.getArtworkMrl();
             if (TextUtils.isEmpty(mw.getArtworkMrl())) {
-                mBinding.songsList.setBackgroundResource(DEFAULT_BACKGROUND_ID);
-                mBinding.backgroundView.setVisibility(View.INVISIBLE);
+                setDefaultBackground();
             } else {
                 VLCApplication.runBackground(new Runnable() {
                     @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
@@ -302,6 +311,8 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
                                     mBinding.songsList.setBackgroundResource(0);
                                 }
                             });
+                        else
+                            setDefaultBackground();
                     }
                 });
             }
@@ -311,6 +322,11 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
 
     }
 
+    private void setDefaultBackground() {
+        mBinding.songsList.setBackgroundResource(DEFAULT_BACKGROUND_ID);
+        mBinding.backgroundView.setVisibility(View.INVISIBLE);
+    }
+
     public void updateList() {
         hideSearchField();
         if (mService != null)
@@ -402,10 +418,16 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
 
     public void onPlaylistSwitchClick(View view) {
         boolean showCover = mBinding.songsList.getVisibility() == View.VISIBLE;
-        mBinding.playlistSwitch.setImageResource(UiTools.getResourceFromAttribute(view.getContext(),
-                showCover ? R.attr.ic_playlist : R.attr.ic_playlist_on));
-        mBinding.songsList.setVisibility(showCover ? View.GONE : View.VISIBLE);
-        mBinding.coverMediaSwitcher.setVisibility(showCover ? View.VISIBLE : View.GONE);
+        if (playlistConstraintSet != null) {
+            TransitionManager.beginDelayedTransition(mBinding.contentLayout);
+            ConstraintSet cs = showCover ? coverConstraintSet : playlistConstraintSet;
+            cs.applyTo(mBinding.contentLayout);
+            mBinding.playlistSwitch.setImageResource(UiTools.getResourceFromAttribute(view.getContext(),
+                    showCover ? R.attr.ic_playlist : R.attr.ic_playlist_on));
+        } else {
+            mBinding.songsList.setVisibility(showCover ? View.GONE : View.VISIBLE);
+            mBinding.coverMediaSwitcher.setVisibility(showCover ? View.VISIBLE : View.GONE);
+        }
     }
 
     public void onShuffleClick(View view) {
@@ -421,8 +443,9 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
         }
     }
 
+
     public void showAdvancedOptions(View v) {
-        if (!isResumed())
+        if (!isVisible())
             return;
         FragmentManager fm = getActivity().getSupportFragmentManager();
         AdvOptionsDialog advOptionsDialog = new AdvOptionsDialog();
