@@ -66,12 +66,12 @@ class StorageBrowserAdapter extends BaseBrowserAdapter {
             vh.setContextMenuListener();
     }
 
-    public void addItem(MediaLibraryItem item, boolean top) {
+    public void addItem(MediaLibraryItem item, boolean top, int position) {
         if (item.getItemType() == MediaLibraryItem.TYPE_MEDIA)
              item = new Storage(((MediaWrapper)item).getUri());
         else if (item.getItemType() != MediaLibraryItem.TYPE_STORAGE)
             return;
-        super.addItem(item, top);
+        super.addItem(item, top, position);
     }
 
     private void removeDir(final String path) {
@@ -101,9 +101,9 @@ class StorageBrowserAdapter extends BaseBrowserAdapter {
     }
 
     protected void openMediaFromView(MediaViewHolder holder, View v) {
-        MediaWrapper mw = new MediaWrapper(((Storage) getItem(holder.getAdapterPosition())).getUri());
+        MediaWrapper mw = new MediaWrapper(((Storage) getItem(holder.getLayoutPosition())).getUri());
         mw.setType(MediaWrapper.TYPE_DIR);
-        fragment.browse(mw, holder.getAdapterPosition(), holder.binding.browserCheckbox.isChecked());
+        fragment.browse(mw, holder.getLayoutPosition(), holder.binding.browserCheckbox.isChecked());
     }
 
     protected void checkBoxAction(View v, String path){
