@@ -34,6 +34,7 @@ import android.content.res.TypedArray;
 import android.graphics.Color;
 import android.os.Build;
 import android.os.Bundle;
+import android.os.Parcelable;
 import android.support.v4.app.DialogFragment;
 import android.support.v4.content.ContextCompat;
 import android.support.v7.preference.PreferenceManager;
@@ -230,7 +231,7 @@ public class AdvOptionsDialog extends DialogFragment implements View.OnClickList
             case ID_SAVE_PLAYLIST:
                 newFragment = new SavePlaylistDialog();
                 Bundle args = new Bundle();
-                args.putParcelableArrayList(SavePlaylistDialog.KEY_TRACKS, mService.getMedias());
+                args.putParcelableArray(SavePlaylistDialog.KEY_TRACKS, (Parcelable[]) mService.getMedias().toArray());
                 newFragment.setArguments(args);
                 tag = "fragment_save_playlist";
                 break;
@@ -558,7 +559,7 @@ public class AdvOptionsDialog extends DialogFragment implements View.OnClickList
                 mAdapter.addOption(new Option(ID_PLAY_AS_AUDIO, R.attr.ic_playasaudio_on, getString(R.string.play_as_audio)));
             mAdapter.addOption(new Option(ID_SPU_DELAY, R.attr.ic_subtitledelay, getString(R.string.spu_delay)));
             mAdapter.addOption(new Option(ID_AUDIO_DELAY, R.attr.ic_audiodelay, getString(R.string.audio_delay)));
-            if (!tvUi || AndroidDevices.isAndroidTv() && AndroidUtil.isNougatOrLater())
+            if (!tvUi || AndroidDevices.isAndroidTv() && AndroidUtil.isNougatOrLater)
                 mAdapter.addOption(new Option(ID_POPUP_VIDEO, R.attr.ic_popup_dim, getString(R.string.popup_playback_title)));
             mAdapter.addOption(new Option(ID_REPEAT, R.attr.ic_repeat, getString(R.string.repeat_title)));
             if (mService.canShuffle())
@@ -704,7 +705,7 @@ public class AdvOptionsDialog extends DialogFragment implements View.OnClickList
 
             @Override
             public boolean onLongClick(View view) {
-                Toast toast = Toast.makeText(VLCApplication.getAppContext(), mList.get(getAdapterPosition()).text, Toast.LENGTH_SHORT);
+                Toast toast = Toast.makeText(VLCApplication.getAppContext(), mList.get(getLayoutPosition()).text, Toast.LENGTH_SHORT);
                 toast.setGravity(Gravity.TOP,0,0);
                 toast.show();
                 return true;
