@@ -245,10 +245,10 @@ public class UiTools {
         });
     }
 
-    public static void addToPlaylist(FragmentActivity activity, ArrayList<MediaWrapper> tracks) {
+    public static void addToPlaylist(FragmentActivity activity, MediaWrapper[] tracks) {
         SavePlaylistDialog savePlaylistDialog = new SavePlaylistDialog();
         Bundle args = new Bundle();
-        args.putParcelableArrayList(SavePlaylistDialog.KEY_NEW_TRACKS, tracks);
+        args.putParcelableArray(SavePlaylistDialog.KEY_NEW_TRACKS, tracks);
         savePlaylistDialog.setArguments(args);
         savePlaylistDialog.show(activity.getSupportFragmentManager(), "fragment_add_to_playlist");
     }
@@ -273,8 +273,8 @@ public class UiTools {
     }
 
     @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR1)
-    public static Bitmap blurBitmap(Bitmap bitmap){
-        if (bitmap == null)
+    public static Bitmap blurBitmap(Bitmap bitmap) {
+        if (bitmap == null || bitmap.getConfig() == null)
             return null;
 
 		//Let's create an empty bitmap with the same size of the bitmap we want to blur
