@@ -33,18 +33,18 @@ import android.os.Handler;
 import android.os.Looper;
 import android.support.v17.leanback.widget.ImageCardView;
 import android.support.v17.leanback.widget.Presenter;
+import android.support.v4.content.ContextCompat;
 import android.text.TextUtils;
 import android.view.View;
 import android.view.ViewGroup;
 import android.widget.ImageView;
 
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.helpers.AsyncImageLoader;
 import org.videolan.vlc.gui.helpers.AudioUtil;
-import org.videolan.vlc.gui.helpers.BitmapUtil;
-import org.videolan.vlc.gui.tv.browser.MusicFragment;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.HttpImageLoader;
 
 @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
@@ -62,7 +62,7 @@ public class CardPresenter extends Presenter {
     public CardPresenter(Activity context){
         mContext = context;
         mRes = mContext.getResources();
-        sDefaultCardImage = mRes.getDrawable(R.drawable.ic_no_thumbnail_big);
+        sDefaultCardImage = ContextCompat.getDrawable(mContext, R.drawable.ic_no_thumbnail_big);
         CARD_WIDTH = mRes.getDimensionPixelSize(R.dimen.tv_grid_card_thumb_width);
         CARD_HEIGHT = mRes.getDimensionPixelSize(R.dimen.tv_grid_card_thumb_height);
     }
@@ -73,19 +73,18 @@ public class CardPresenter extends Presenter {
         public ViewHolder(View view) {
             super(view);
             mCardView = (ImageCardView) view;
+            mCardView.getMainImageView().setScaleType(ImageView.ScaleType.FIT_CENTER);
         }
 
-        protected void updateCardViewImage(MediaWrapper mediaWrapper) {
-            mCardView.getMainImageView().setScaleType(ImageView.ScaleType.FIT_CENTER);
-            if (!TextUtils.isEmpty(mediaWrapper.getArtworkURL()) && mediaWrapper.getArtworkURL().startsWith("http")) {
-                AsyncImageLoader.LoadImage(new HttpImageLoader(mediaWrapper.getArtworkURL()), mCardView);
+        void updateCardViewImage(MediaLibraryItem mediaLibraryItem) {
+            if (!TextUtils.isEmpty(mediaLibraryItem.getArtworkMrl()) && mediaLibraryItem.getArtworkMrl().startsWith("http")) {
+                AsyncImageLoader.LoadImage(new HttpImageLoader(mediaLibraryItem.getArtworkMrl()), mCardView);
             } else {
-                AsyncImageLoader.LoadImage(new CoverFetcher(mediaWrapper), mCardView);
+                AsyncImageLoader.LoadImage(new CoverFetcher(mediaLibraryItem), mCardView);
             }
         }
 
-        protected void updateCardViewImage(Drawable image) {
-            mCardView.getMainImageView().setScaleType(ImageView.ScaleType.FIT_CENTER);
+        void updateCardViewImage(Drawable image) {
             mCardView.setMainImage(image);
         }
     }
@@ -96,7 +95,7 @@ public class CardPresenter extends Presenter {
         ImageCardView cardView = new ImageCardView(mContext);
         cardView.setFocusable(true);
         cardView.setFocusableInTouchMode(true);
-        cardView.setBackgroundColor(mRes.getColor(R.color.lb_details_overview_bg_color));
+        cardView.setBackgroundColor(ContextCompat.getColor(mContext, R.color.lb_details_overview_bg_color));
         cardView.setMainImageDimensions(CARD_WIDTH, CARD_HEIGHT);
         return new ViewHolder(cardView);
     }
@@ -108,9 +107,8 @@ public class CardPresenter extends Presenter {
             final MediaWrapper mediaWrapper = (MediaWrapper) item;
             holder.mCardView.setTitleText(mediaWrapper.getTitle());
             holder.mCardView.setContentText(mediaWrapper.getDescription());
-            if (mediaWrapper.getType() == mediaWrapper.TYPE_GROUP)
-                holder.updateCardViewImage(mRes.getDrawable(
-                        R.drawable.ic_video_collection_big));
+            if (mediaWrapper.getType() == MediaWrapper.TYPE_GROUP)
+                holder.updateCardViewImage(ContextCompat.getDrawable(mContext, R.drawable.ic_video_collection_big));
             else
                 holder.updateCardViewImage(mediaWrapper);
             holder.view.setOnLongClickListener(new View.OnLongClickListener() {
@@ -120,19 +118,18 @@ public class CardPresenter extends Presenter {
                     return true;
                 }
             });
-        } else if (item instanceof MusicFragment.ListItem) {
-            MusicFragment.ListItem listItem = (MusicFragment.ListItem) item;
-            MediaWrapper MediaWrapper = listItem.mediaList.get(0);
-            holder.mCardView.setTitleText(listItem.mTitle);
-            holder.mCardView.setContentText(listItem.mSubTitle);
-            holder.updateCardViewImage(MediaWrapper);
+        } else if (item instanceof MediaLibraryItem) {
+            MediaLibraryItem mediaLibraryItem = (MediaLibraryItem) item;
+            holder.mCardView.setTitleText(mediaLibraryItem.getTitle());
+            holder.mCardView.setContentText(mediaLibraryItem.getDescription());
+            holder.updateCardViewImage(mediaLibraryItem);
         } else if (item instanceof SimpleCard){
             SimpleCard card = (SimpleCard) item;
             Bitmap image = card.getImage();
             holder.mCardView.setTitleText(card.getName());
-            holder.mCardView.setContentText("");
-            holder.updateCardViewImage(image != null ? new BitmapDrawable(image) : mRes.getDrawable(card.getImageId()));
-        }else if (item instanceof String){
+            holder.mCardView.setContentText(card.getDescription());
+            holder.updateCardViewImage(image != null ? new BitmapDrawable(mRes, image) : ContextCompat.getDrawable(mContext, card.getImageId()));
+        } else if (item instanceof String){
             holder.mCardView.setTitleText((String) item);
             holder.mCardView.setContentText("");
             holder.updateCardViewImage(sDefaultCardImage);
@@ -148,10 +145,11 @@ public class CardPresenter extends Presenter {
         // TODO?
     }
 
-    public static class SimpleCard {
+    static class SimpleCard {
         long id;
         int imageId;
         String name;
+        String description;
         Bitmap image;
 
         Uri uri;
@@ -165,6 +163,14 @@ public class CardPresenter extends Presenter {
         SimpleCard(long id, String name, int imageId){
             this.id = id;
             this.name = name;
+            this.description = "";
+            this.imageId = imageId;
+        }
+
+        SimpleCard(long id, String name, String description, int imageId){
+            this.id = id;
+            this.name = name;
+            this.description = description;
             this.imageId = imageId;
         }
 
@@ -210,39 +216,58 @@ public class CardPresenter extends Presenter {
             return name;
         }
 
+        public String getDescription() {
+            return description;
+        }
+
         public void setName(String name) {
             this.name = name;
         }
     }
 
-    public static class CoverFetcher implements AsyncImageLoader.Callbacks{
-        MediaWrapper mediaWrapper;
+    private static class CoverFetcher implements AsyncImageLoader.Callbacks{
+        MediaLibraryItem mediaLibraryItem;
         private static Resources res;
 
-        CoverFetcher(MediaWrapper mediaWrapper){
-            this.mediaWrapper = mediaWrapper;
+        CoverFetcher(MediaLibraryItem mediaLibraryItem){
+            this.mediaLibraryItem = mediaLibraryItem;
             res = VLCApplication.getAppResources();
         }
 
         @Override
         public Bitmap getImage() {
-            Bitmap picture;
-            if (mediaWrapper.getType() == mediaWrapper.TYPE_AUDIO) {
-                picture = AudioUtil.getCover(VLCApplication.getAppContext(), mediaWrapper, 320);
-                if (picture == null)
-                    picture = BitmapFactory.decodeResource(res, R.drawable.ic_browser_audio_big_normal);
-            } else if (mediaWrapper.getType() == mediaWrapper.TYPE_VIDEO) {
-                picture = BitmapUtil.getPicture(mediaWrapper);
-                if (picture == null)
-                    picture = BitmapFactory.decodeResource(res, R.drawable.ic_browser_video_big_normal);
-            } else if (mediaWrapper.getType() == mediaWrapper.TYPE_DIR) {
-                if (TextUtils.equals(mediaWrapper.getUri().getScheme(),"file"))
+            Bitmap picture = null;
+            if (mediaLibraryItem.getItemType() == MediaLibraryItem.TYPE_MEDIA && ((MediaWrapper) mediaLibraryItem).getType() == MediaWrapper.TYPE_DIR) {
+                MediaWrapper mediaWrapper = (MediaWrapper) mediaLibraryItem;
+                if (TextUtils.equals(mediaWrapper.getUri().getScheme(), "file"))
                     picture = BitmapFactory.decodeResource(res, R.drawable.ic_menu_folder_big);
                 else
                     picture = BitmapFactory.decodeResource(res, R.drawable.ic_menu_network_big);
+            } else
+                picture = AudioUtil.readCoverBitmap(Uri.decode(mediaLibraryItem.getArtworkMrl()), res.getDimensionPixelSize(R.dimen.tv_grid_card_thumb_width));
+            if (picture == null) {
+                int resId;
+                switch (mediaLibraryItem.getItemType()) {
+                    case MediaLibraryItem.TYPE_ALBUM:
+                        resId = R.drawable.ic_album_big;
+                        break;
+                    case MediaLibraryItem.TYPE_ARTIST:
+                        resId = R.drawable.ic_artist_big;
+                        break;
+                    case MediaLibraryItem.TYPE_GENRE:
+                        resId = R.drawable.ic_genre_big;
+                        break;
+                    case MediaLibraryItem.TYPE_MEDIA:
+                        if (((MediaWrapper)mediaLibraryItem).getType() == MediaWrapper.TYPE_VIDEO)
+                            resId = R.drawable.ic_browser_video_big_normal;
+                        else
+                            resId = R.drawable.ic_song_big;
+                        break;
+                    default:
+                        resId = R.drawable.ic_browser_unknown_big_normal;
+                }
+                picture = BitmapFactory.decodeResource(res, resId);
             }
-            else
-                picture = BitmapFactory.decodeResource(res, R.drawable.ic_browser_unknown_big_normal);
             return picture;
         }
 
