@@ -20,12 +20,17 @@
  *****************************************************************************/
 package org.videolan.vlc.gui.tv.audioplayer;
 
+import android.annotation.TargetApi;
 import android.content.SharedPreferences;
 import android.graphics.Bitmap;
+import android.net.Uri;
+import android.os.Build;
 import android.os.Bundle;
 import android.preference.PreferenceManager;
+import android.support.v7.widget.DividerItemDecoration;
 import android.support.v7.widget.LinearLayoutManager;
 import android.support.v7.widget.RecyclerView;
+import android.text.TextUtils;
 import android.view.InputDevice;
 import android.view.KeyEvent;
 import android.view.MotionEvent;
@@ -36,21 +41,21 @@ import android.widget.TextView;
 
 import org.videolan.libvlc.Media;
 import org.videolan.libvlc.MediaPlayer;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
+import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.MediaComparators;
+import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.preferences.PreferencesActivity;
 import org.videolan.vlc.gui.tv.browser.BaseTvActivity;
-import org.videolan.vlc.gui.view.DividerItemDecoration;
-import org.videolan.vlc.media.MediaLibrary;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
-import org.videolan.vlc.util.Util;
 
 import java.util.ArrayList;
 import java.util.Collections;
 
+@TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
 public class AudioPlayerActivity extends BaseTvActivity implements PlaybackService.Client.Callback,
         PlaybackService.Callback, View.OnFocusChangeListener {
     public static final String TAG = "VLC/AudioPlayerActivity";
@@ -68,9 +73,10 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
     private long mLastMove;
     private int mCurrentlyPlaying, mPositionSaved = 0;
     private boolean mShuffling = false;
+    private String mCurrentCoverArt;
 
     private TextView mTitleTv, mArtistTv;
-    private ImageView mPlayPauseButton, mCover, mNext, mShuffle, mRepeat;
+    private ImageView mPlayPauseButton, mCover, mNext, mShuffle, mRepeat, mBackground;
     private ProgressBar mProgressBar;
 
     protected void onCreate(Bundle savedInstanceState) {
@@ -82,10 +88,10 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
         mRecyclerView = (RecyclerView) findViewById(R.id.playlist);
         mLayoutManager = new LinearLayoutManager(this);
         mRecyclerView.setLayoutManager(mLayoutManager);
-        mRecyclerView.addItemDecoration(new DividerItemDecoration(this, DividerItemDecoration.VERTICAL_LIST));
+        mRecyclerView.addItemDecoration(new DividerItemDecoration(this, DividerItemDecoration.VERTICAL));
         mRecyclerView.setOnFocusChangeListener(this);
         if (mMediaList == null)
-            mMediaList = new ArrayList<MediaWrapper>();
+            mMediaList = new ArrayList<>();
 //        if (getIntent().getData() != null)
 //            mMediaList.add(getIntent().getDataString());
         mAdapter = new PlaylistAdapter(this, mMediaList);
@@ -99,6 +105,7 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
         mRepeat = (ImageView)findViewById(R.id.button_repeat);
         mProgressBar = (ProgressBar)findViewById(R.id.media_progress);
         mCover = (ImageView)findViewById(R.id.album_cover);
+        mBackground = (ImageView)findViewById(R.id.background);
     }
 
 
@@ -130,6 +137,8 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
             mService.load(mMediaList, mCurrentlyPlaying);
         } else {
             mMediaList = medias;
+            if (mCurrentlyPlaying != mService.getCurrentMediaPosition())
+                mService.playIndex(mCurrentlyPlaying);
             update();
             mAdapter = new PlaylistAdapter(this, mMediaList);
             mRecyclerView.setAdapter(mAdapter);
@@ -154,8 +163,9 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
         mPlayPauseButton.setImageResource(mService.isPlaying() ? R.drawable.ic_pause_w : R.drawable.ic_play_w);
         if (mService.hasMedia()) {
             SharedPreferences mSettings= PreferenceManager.getDefaultSharedPreferences(this);
-            if (mSettings.getBoolean(PreferencesActivity.VIDEO_RESTORE, false)){
-                Util.commitPreferences(mSettings.edit().putBoolean(PreferencesActivity.VIDEO_RESTORE, false));
+            if (mSettings.getBoolean(PreferencesActivity.VIDEO_RESTORE, false)) {
+                mSettings.edit().putBoolean(PreferencesActivity.VIDEO_RESTORE, false).apply();
+                mService.getCurrentMediaWrapper().removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
                 mService.switchToVideo();
                 finish();
                 return;
@@ -163,17 +173,33 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
             mTitleTv.setText(mService.getTitle());
             mArtistTv.setText(mService.getArtist());
             mProgressBar.setMax((int) mService.getLength());
-            MediaWrapper MediaWrapper = MediaLibrary.getInstance().getMediaItem(mService.getCurrentMediaLocation());
-            Bitmap cover = AudioUtil.getCover(this, MediaWrapper, mCover.getWidth());
-            if (cover == null)
-                cover = mService.getCover();
-            if (cover == null)
-                mCover.setImageResource(R.drawable.ic_tv_icon_big);
-            else
-                mCover.setImageBitmap(cover);
-
-            mCurrentlyPlaying=mService.getCurrentMediaPosition();
+            mCurrentlyPlaying = mService.getCurrentMediaPosition();
             selectItem(mCurrentlyPlaying);
+            final MediaWrapper mw = mService.getCurrentMediaWrapper();
+            if (TextUtils.equals(mCurrentCoverArt, mw.getArtworkMrl()))
+                return;
+            mCurrentCoverArt = mw.getArtworkMrl();
+            VLCApplication.runBackground(new Runnable() {
+                @Override
+                public void run() {
+                    final Bitmap cover = AudioUtil.readCoverBitmap(Uri.decode(mCurrentCoverArt), mCover.getWidth());
+                    final Bitmap blurredCover = UiTools.blurBitmap(cover);
+                    VLCApplication.runOnMainThread(new Runnable() {
+                        @Override
+                        public void run() {
+                            if (cover == null) {
+                                mCover.setImageResource(R.drawable.ic_tv_icon_big);
+                                mBackground.clearColorFilter();
+                                mBackground.setImageResource(0);
+                            } else {
+                                mCover.setImageBitmap(cover);
+                                mBackground.setColorFilter(UiTools.getColorFromAttribute(mBackground.getContext(), R.attr.audio_player_background_tint));
+                                mBackground.setImageBitmap(blurredCover);
+                            }
+                        }
+                    });
+                }
+            });
         }
     }
 
@@ -357,7 +383,7 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
 
     private void goPrevious() {
         if (mService != null && mService.hasPrevious()) {
-            mService.previous();
+            mService.previous(false);
         }
     }
 
